!    ********************************************************
      subroutine alphaTest(startAlpha)

      use shared
      implicit none

      double precision, intent(in) :: startAlpha

      integer, parameter :: npt = 14

      double precision, allocatable, dimension(:) :: x, y, hdir, grad, gradp

      double precision :: costval, costvalp, alpha, testval, prod

      integer :: i, iflag, ierr

! allocate local variables
	      allocate (x(nctr), STAT=ierr)
	      if(ierr /= 0) write(*,*) "allocation error for x"

	      allocate (y(nctr), STAT=ierr)
	      if(ierr /= 0) write(*,*) "allocation error for y"

	      allocate (hdir(nctr), STAT=ierr)
	      if(ierr /= 0) write(*,*) "allocation error for hdir"

	      allocate (grad(nctr), STAT=ierr)
	      if(ierr /= 0) write(*,*) "allocation error for grad"

	      allocate (gradp(nctr), STAT=ierr)
	      if(ierr /= 0) write(*,*) "allocation error for dummygrad"

! execute alpha test
          x = 0.

          call funder(nctr,x,costval,grad,iflag)

          print *,'Original cost:',costval,' grad norm:',sqrt(dot_product(grad,grad))

          hdir = grad / sqrt(dot_product(grad,grad))

          ! h^T grad
          prod = dot_product(grad,hdir)

          alpha = startAlpha

          do i = 1,npt

              y = x + alpha*hdir

              call funder(nctr,y,costvalp,gradp,iflag)

              ! \frac{J(x + \alpha h) - J(x)}{\alpha h^T grad}
              testval = (costvalp - costval) / (alpha*prod)

              print *,'i=',i,'alpha = ',alpha,', alpha test:',testval, &
                  'O(alpha):',(1-testval)/alpha,' cost:',costvalp,' grad norm:',sqrt(dot_product(gradp,gradp))

              alpha = alpha/10.
          enddo

! deallocate local variables
          deallocate (x)
          deallocate (y)
          deallocate (hdir)
          deallocate (grad)
          deallocate (gradp)

          return
      end
