module mlef_mod
    use shared, only : NoMembers, n, m, nt, ns, nobs, DX, DY, CovLocalizationLength, &
                       alloc1, alloc2, alloc3, alloc4

    implicit none

    double precision,DIMENSION(:,:,:,:),allocatable :: EnsembleU
    double precision,DIMENSION(:,:,:,:),allocatable :: EnsembleV
    double precision,DIMENSION(:,:,:,:),allocatable :: EnsemblePhi

    double precision,DIMENSION(:,:),allocatable :: data2DPhi
    double precision,DIMENSION(:,:),allocatable :: data2DU
    double precision,DIMENSION(:,:),allocatable :: data2DV

    ! X_p, X_f: N_{state} \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: perturbationU
    double precision,DIMENSION(:,:),allocatable :: perturbationV
    double precision,DIMENSION(:,:),allocatable :: perturbationPhi

    ! Y_f, Y_p: N_{obs} \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: ensObsU
    double precision,DIMENSION(:,:),allocatable :: ensObsV
    double precision,DIMENSION(:,:),allocatable :: ensObsPhi

    ! \bar{x}: N_{state}
    double precision,DIMENSION(:),allocatable :: meanU
    double precision,DIMENSION(:),allocatable :: meanV
    double precision,DIMENSION(:),allocatable :: meanPhi

    ! \bar{y}_f: N_{obs}
    double precision,DIMENSION(:),allocatable :: meanObsU
    double precision,DIMENSION(:),allocatable :: meanObsV
    double precision,DIMENSION(:),allocatable :: meanObsPhi

    ! X_p^\ell: N_{state}^\ell \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: localPerturbationU
    double precision,DIMENSION(:,:),allocatable :: localPerturbationV
    double precision,DIMENSION(:,:),allocatable :: localPerturbationPhi

    ! \bar{x}_f^\ell: N_{state}^\ell
    double precision,DIMENSION(:),allocatable :: localMeanU
    double precision,DIMENSION(:),allocatable :: localMeanV
    double precision,DIMENSION(:),allocatable :: localMeanPhi

    ! Y_p^\ell: N_{obs}^\ell \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: localEnsObsU
    double precision,DIMENSION(:,:),allocatable :: localEnsObsV
    double precision,DIMENSION(:,:),allocatable :: localEnsObsPhi

    ! \bar{y}_f^\ell: N_{obs}^\ell
    double precision,DIMENSION(:),allocatable :: localMeanObsU
    double precision,DIMENSION(:),allocatable :: localMeanObsV
    double precision,DIMENSION(:),allocatable :: localMeanObsPhi

    ! C: N_{ens} \times N_{obs}^\ell
    double precision,DIMENSION(:,:),allocatable :: localTempEnsObsU
    double precision,DIMENSION(:,:),allocatable :: localTempEnsObsV
    double precision,DIMENSION(:,:),allocatable :: localTempEnsObsPhi

    ! \tilde{P}_a: N_{ens} \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: localAnalysisCovarU
    double precision,DIMENSION(:,:),allocatable :: localAnalysisCovarV
    double precision,DIMENSION(:,:),allocatable :: localAnalysisCovarPhi

    ! D: N_{ens} -- eigenvalues of \tilde{P}_a = L D L^T
    double precision,DIMENSION(:),allocatable :: localEigvU
    double precision,DIMENSION(:),allocatable :: localEigvV
    double precision,DIMENSION(:),allocatable :: localEigvPhi

    ! work: N_{work} -- work matrix for lapack dsyev
    double precision,DIMENSION(:),allocatable :: localWork

    ! W_a: N_{ens} \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: localTempEnsEnsU
    double precision,DIMENSION(:,:),allocatable :: localTempEnsEnsV
    double precision,DIMENSION(:,:),allocatable :: localTempEnsEnsPhi

    ! intermediate var: N_{ens} \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: localTempEnsEns2U
    double precision,DIMENSION(:,:),allocatable :: localTempEnsEns2V
    double precision,DIMENSION(:,:),allocatable :: localTempEnsEns2Phi

    ! \bar{w}: N_{ens}
    double precision,DIMENSION(:),allocatable :: localMeanTempEnsEnsU
    double precision,DIMENSION(:),allocatable :: localMeanTempEnsEnsV
    double precision,DIMENSION(:),allocatable :: localMeanTempEnsEnsPhi

    ! X_a^\ell: N_{state}^\ell \times N_{ens}
    double precision,DIMENSION(:,:),allocatable :: localAnalysisU
    double precision,DIMENSION(:,:),allocatable :: localAnalysisV
    double precision,DIMENSION(:,:),allocatable :: localAnalysisPhi

    integer :: NXCorrLength, NYCorrLength, analysisPoint, analysisI, analysisJ, &
               localn, minvar

contains

    subroutine mlef_initialize()

        implicit none

	    ! Calculate maximum local size
	    NXCorrLength = ceiling(CovLocalizationLength/DX)
	    NYCorrLength = ceiling(CovLocalizationLength/DY)

	    ! X: N_{ens} \times n \times m \times nt
	    call alloc4(EnsembleU,NoMembers,n,m,nt,'EnsembleU')
	    call alloc4(EnsembleV,NoMembers,n,m,nt,'EnsembleV')
	    call alloc4(EnsemblePhi,NoMembers,n,m,nt,'EnsemblePhi')

	    ! helper data: n-2 \times m-2
	    call alloc2(data2DU,n-2,m-2,'data2DU')
	    call alloc2(data2DV,n-2,m-2,'data2DV')
	    call alloc2(data2DPhi,n-2,m-2,'data2DPhi')

	    ! X_p, X_f: N_{state} \times N_{ens}
	    call alloc2(perturbationU,ns,NoMembers,'perturbationU')
	    call alloc2(perturbationV,ns,NoMembers,'perturbationV')
	    call alloc2(perturbationPhi,ns,NoMembers,'perturbationPhi')

	    ! Y_f, Y_p: N_{obs} \times N_{ens}
	    call alloc2(ensObsU,nobs,NoMembers,'ensObsU')
	    call alloc2(ensObsV,nobs,NoMembers,'ensObsV')
	    call alloc2(ensObsPhi,nobs,NoMembers,'ensObsPhi')

	    ! \bar{x}: N_{state}
	    call alloc1(meanU,ns,'meanU')
	    call alloc1(meanV,ns,'meanV')
	    call alloc1(meanPhi,ns,'meanPhi')

	    ! \bar{y}_f: N_{obs}
	    call alloc1(meanObsU,nobs,'meanObsU')
	    call alloc1(meanObsV,nobs,'meanObsV')
	    call alloc1(meanObsPhi,nobs,'meanObsPhi')

	    ! \tilde{P}_a: N_{ens} \times N_{ens}
	    call alloc2(localAnalysisCovarU,NoMembers,NoMembers,'localAnalysisCovarU')
	    call alloc2(localAnalysisCovarV,NoMembers,NoMembers,'localAnalysisCovarV')
	    call alloc2(localAnalysisCovarPhi,NoMembers,NoMembers,'localAnalysisCovarPhi')

	    ! D: N_{ens}
	    call alloc1(localEigvU,NoMembers,'localEigvU')
	    call alloc1(localEigvV,NoMembers,'localEigvV')
	    call alloc1(localEigvPhi,NoMembers,'localEigvPhi')

	    ! W_a: N_{ens} \times N_{ens}
	    call alloc2(localTempEnsEnsU,NoMembers,NoMembers,'localTempEnsEnsU')
	    call alloc2(localTempEnsEnsV,NoMembers,NoMembers,'localTempEnsEnsV')
	    call alloc2(localTempEnsEnsPhi,NoMembers,NoMembers,'localTempEnsEnsPhi')

	    ! temp: N_{ens} \times N_{ens}
	    call alloc2(localTempEnsEns2U,NoMembers,NoMembers,'localTempEnsEns2U')
	    call alloc2(localTempEnsEns2V,NoMembers,NoMembers,'localTempEnsEns2V')
	    call alloc2(localTempEnsEns2Phi,NoMembers,NoMembers,'localTempEnsEns2Phi')

	    ! \bar{w}: N_{ens}
	    call alloc1(localMeanTempEnsEnsU,NoMembers,'localMeanTempEnsEnsU')
	    call alloc1(localMeanTempEnsEnsV,NoMembers,'localMeanTempEnsEnsV')
	    call alloc1(localMeanTempEnsEnsPhi,NoMembers,'localMeanTempEnsEnsPhi')

    end subroutine

endmodule mlef_mod
