      program tswcgdescent
!    **************************************************
!     adjoint system for the shallow-water equations
!     test program for data assimilation with shallow
!     water equations when controlling both boundary
!     and initial condition.
!     *********************************************
!             here commons:  tabx scalx
!     *********************************************
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, k_max = nt,&
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, ns=nm2*mm2, &
        nctr = 3*nm2*mm2, ndv = nctr)
      integer iprint(2)
      logical diagco

      dimension x(nctr),hdir(nctr),y(nctr),grad(nctr),gtemp(nctr), &
      izs(1),xtemp(nctr),iz(22),tabgr(nctr),z(nctr),diag(nctr)

      integer mp,point,status
      double precision dzs(10),apd
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
      common /xlz/cost0
      common /background/ uback(nm2,mm2),vback(nm2,mm2),phiback(nm2,mm2),& 
                          sqrtPfU(nm2*mm2,nm2*mm2),&
                          sqrtPfV(nm2*mm2,nm2*mm2),&
                          sqrtPfPhi(nm2*mm2,nm2*mm2)
      double precision :: DX, DY, CorrH, UBackgroundStddev, & 
                          VBackgroundStddev, PhiBackgroundStddev
      namelist /perturbation/ NoMembers, DX, DY, CorrH, & 
                              UBackgroundStddev, VBackgroundStddev, &
                              PhiBackgroundStddev
      double precision :: UObsStddev, VObsStddev, PhiObsStddev
      namelist /obs_errors/ UObsStddev, VObsStddev, PhiObsStddev
      external myvalue
      external mygrad
      external cgdescent
      double precision temp(nm2*mm2)
      common /exact/ uexact(n,m,nt), vexact(n,m,nt), phiexact(n,m,nt)
!
      open(unit=42,file="namelist.swm",status='old')
      read(42,perturbation)
      close(42)

      open(unit=42,file="namelist.swm",status='old')
      read(42,obs_errors)
      close(42)

! ** *************************************************************
!                  construction of observations
!         main programm : initialization
!                       : getting 'observations'
!                       : calling  simulator and optimizor
!                       : calling  tangent and adjoint
!                         verification routines.

!   **************************************************************
!   initialization of scalar variables of the simulator
      open(unit=7,file='swe7.dat',status='unknown')
      open(unit=8,file='swe8.dat',status='unknown')
      jjvar=7
!      write(jjvar,641)
641   format(1x, 'run no 1 of the 5 february 1990',/, &
             1x, ' non penalized cost function ',/, &
             1x, ' observations at each grid poind and time-step ')

      cost0=1d0
      epnu=0
      epnv=0
      epnphi=0
      ztemp= 300.
      htemp= 2.*ztemp
!      write(jjvar,642)epnu,epnv,epnphi
642   format(1x, 'penalisations:',/ &
       1x, 'epnu = ',e14.7,/, &
       1x, 'epnv = ',e14.7,/, &
       1x, 'epnphi = ',e14.7,//)
      do 12 k=0,nt
      do 12 i=1,n
      do 13 j=1,m
      uobs(i,j,k)=0.
      vobs(i,j,k)= 0.
      phobs(i,j,k)=0.
13    continue
12    continue
      do 32 i=1,n
      do 33 j=1,m
      ! wu(i,j)= 1.d-2 
      ! wv(i,j)= 1.d-2 
      ! wphi(i,j)= 1.d-4 
      wu(i,j)= 1/UObsStddev**2
      wv(i,j)= 1/VObsStddev**2
      wphi(i,j)= 1/PhiObsStddev**3
33    continue
32    continue
!      write(jjvar,643)wu(10,10),wv(10,10),wphi(10,10)
643   format(1x, 'weight:',/ &
       1x, 'wu(i,j) = ',e14.7,/, &
       1x, 'wv(i,j) = ',e14.7,/, &
       1x, 'wphi(i,j)  = ',e14.7,//)
      wu(1,1)=0.
      wv(1,1)=0.
      wphi(1,1)=0.
      wu(1,m)=0.
      wv(1,m)=0.
      wphi(1,m)=0.
      wu(n,1)=0.
      wv(n,1)=0.
      wphi(n,1)=0.
      wu(n,m)=0.
      wv(n,m)=0.
      wphi(n,m)=0.
!   ***************************************************************

!*******************************************************************
!            first integration and first calcul of cost
!                  increasing of the condition initiale
!******************************************************************
!                 activated simulations and outputs:
!*****************************************************************
! preliminars : constructing pseudo- observations and first guess
!  solutions with a grammeltvedt method
!*****************************************************************
      print*,'before initana'
         call initana
         call stdira
         call trsfr(x)
      open(unit=9,file='ini.dat',status='unknown', &
           form='formatted')
      write(9,*) (((u(i,j,k),i=1,n),j=1,m),k=1,nt), &
                 (((v(i,j,k),i=1,n),j=1,m),k=1,nt), &
                 (((phi(i,j,k),i=1,n),j=1,m),k=1,nt)
      close(9)

         call stdir(x,cost)
!         call opngks
!         call outgr(1.,1.)
!         call clsgks
!         call statw(1)
!************************************************************
!************************************************************
!    constructing a perturbation direction
      aui=1.d0
      avi=1.d0
      aphii=1.d-1
      aub=0.d0
      avb=0.d0
      aphib=0.d-1
      ! call dirprt(hdir,aui,avi,aphii,aub,avb,aphib)
!      write(jjvar,644)aui,avi,aphii,aub,avb,aphib
644   format(1x, 'perturbations:',/ &
       1x, 'aui = ',e14.7,/, &
       1x, 'avi = ',e14.7,/, &
       1x, 'aphii = ',e14.7,//, &
       1x, 'aub = ',e14.7,/, &
       1x, 'avb = ',e14.7,/, &
       1x, 'aphib = ',e14.7,//)
      do 4029 i=1,nctr
      x(i)=x(i) + 2.*hdir(i)
!       print 4030,i,x(i),i,hdir(i)
4030   format(1x,'x(',i3,')= ',e14.7,1x, &
      'hdir(',i3,')= ',e14.7)
4029  continue
!********************************************************
!       call simobs
!*********************************************************
!                         washup
      do 2300 k=0,nt
      do 2301 j=1,m
      do 2302 i=1,n
      ui(i,j)= 0.
      vi(i,j)= 0.
      phii(i,j)=0.
      u(i,j,k)= 0.
      v(i,j,k)= 0.
      phi(i,j,k)=0.
      zu(i,j,k)= 0.
      zv(i,j,k)= 0.
      zphi(i,j,k)=0.
2302  continue
2301  continue
2300  continue
5555  continue
      do 5029 i=1,nctr
!      print 5030,i,x(i),i,hdir(i)
5030  format(1x,1x,'x(',i3,')= ',e14.7,1x, &
      'hdir(',i3,')= ',e14.7)
5029  continue
      do 4025 k=0,nt
      do 4026 j=1,m
      do 4027 i=1,n
      u(i,j,k)=0.
      v(i,j,k)=0.
      phi(i,j,k)=0.
      zu(i,j,k)=0.
      zv(i,j,k)=0.
      zphi(i,j,k)=0.
      tu(i,j,k)=0.
      tv(i,j,k)=0.
      tphi(i,j,k)=0.
4027  continue
4026  continue
4025  continue
!**************************************************
!       verifications of linear et adjoint
!                     system
!************************************************
          call stdir(x,hjcou)
!         call opngks
!         call outgr(1.d0,1.d0)
!         call clsgks
      call trsfr(x)
!      open(unit=9,file='ptb.dat',status='unknown',
!     %     form='formatted')
!      write(9,*) ((u(i,j,1),i=1,n),j=1,m),((v(i,j,1),i=1,n),j=1,m),
!     1           ((phi(i,j,1),i=1,n),j=1,m)
!      close(9)
!      if(9.eq.9) stop
!         call opngks
!           call outgr(1.d0,1.d0)
!         call nstgvr(x,hdir,y,nctr)
      open(unit=12,file='HorzCorrU.txt',status='unknown', &
          form='formatted')
      read(12,*) ((sqrtPfU(i,j),i=1,nm2*mm2),j=1,nm2*mm2)
      close(12)
      open(unit=12,file='HorzCorrV.txt',status='unknown', &
          form='formatted')
      read(12,*) ((sqrtPfV(i,j),i=1,nm2*mm2),j=1,nm2*mm2)
      close(12)
      open(unit=12,file='HorzCorrP.txt',status='unknown', &
          form='formatted')
      read(12,*) ((sqrtPfPhi(i,j),i=1,nm2*mm2),j=1,nm2*mm2)
      close(12)

      ensfactor = sqrt(96.)

      sqrtPfU = UBackgroundStddev*sqrtPfU/ensfactor
      sqrtPfV = VBackgroundStddev*sqrtPfV/ensfactor
      sqrtPfPhi = PhiBackgroundStddev*sqrtPfPhi/ensfactor

      open(unit=12,file='SWM_mean.txt',status='unknown', & 
           form='formatted') 
      read(12,*) ((uback(i,j),i=1,nm2),j=1,mm2), & 
                 ((vback(i,j),i=1,nm2),j=1,mm2), &
                 ((phiback(i,j),i=1,nm2),j=1,mm2)
      close(12)

! read observations from file
      open(unit=10,file='SWM_obs.dat',status='unknown', &
           form='formatted')
      read(10,*) (((uobs(i,j,k),i=2,n-1),j=2,m-1),k=1,k_max), &
                 (((vobs(i,j,k),i=2,n-1),j=2,m-1),k=1,k_max), &
                 (((phobs(i,j,k),i=2,n-1),j=2,m-1),k=1,k_max)
      close(10)
      open(unit=10,file='SWM_exact.txt',status='unknown', &
           form='formatted')
      read(10,*) (((uexact(i,j,k),i=1,n),j=1,m),k=1,k_max), &
                 (((vexact(i,j,k),i=1,n),j=1,m),k=1,k_max), &
                 (((phiexact(i,j,k),i=1,n),j=1,m),k=1,k_max)
      close(10)

!      do i=2,nm1
!          do j=2,mm1
!              u(i,j,1) = uback(i-1,j-1)
!              v(i,j,1) = vback(i-1,j-1)
!              phi(i,j,1) = phiback(i-1,j-1)
!          enddo
!      enddo
 
!      call trsfr(x)

      print*,'before ngdvr'
           call ngdvr(x,hdir,y,grad,nctr)
!      if(9.eq.9) stop
!          call clsgks

!***************************************************
!      performing the optimization method
!**************************************************
      isaut=0
      if(isaut.eq.1)go to 9999
!**********************************************************
! i: numerical values choosen for the parameters of the
!                     optimizer
! ********************************************************
      jsimul=0
      nrz=ntrav
      imp=4
      io=7
      i1=7
      niter=30
      nsim=40
      nemax=30
!****************************************************
!         ii: calling the simulator
!**************************************************
      indic=4
      ndim=nctr
      art=3.
      df1=hjcou/art
!***************************************************
!         iii: calling the optimizer
!***************************************************
      isaut=0
      if(isaut.eq.1)go to 9999

      itcomp=0
      lp=6
      mp=6
      diagco=.false.
      iprint(1)=1
      iprint(2)=0
      eps=5.0d0
      point=0.
      iflag=0
      gtol=1.0d-3

      icall=0
 
      do i=2,nm1
          do j=2,mm1
              u(i,j,1) = uback(i-1,j-1)
              v(i,j,1) = vback(i-1,j-1)
              phi(i,j,1) = phiback(i-1,j-1)
          enddo
      enddo
 
      call trsfr(x)

      call stdir(x,hjcou)

      print *,'Initial cost is:',hjcou

      print *,'Initial RMSE is:'

      rmseroot = sqrt(1.*nm2*mm2)

      do k=1,nt
          urmse = 0.
          vrmse = 0.
          phirmse = 0.

          do j=2,mm1
              do i=2,nm1
                  urmse = urmse + (u(i,j,k) - uexact(i,j,k))**2
                  vrmse = vrmse + (v(i,j,k) - vexact(i,j,k))**2
                  phirmse = phirmse + (phi(i,j,k) - phiexact(i,j,k))**2
              enddo
          enddo
          print *,'RMSE(',k,'):', sqrt(urmse)/rmseroot, &
                                  sqrt(vrmse)/rmseroot, &
                                  sqrt(phirmse)/rmseroot
      enddo
      z = 0

      indic=4

      status = 0
      gnorm = 0.
      iter = 0
      nfunc = 0
      ngrad = 0

      call cg_descent (1.d-8, z, ndim, myvalue, mygrad, status, gnorm, &
                       hjcou, iter, nfunc, ngrad, hdir, grad, xtemp, & 
                       gtemp)

!*************************************************
!      calling graphical outputs
!*************************************************
!       call opngks
!       call outgr(1.,1.)
!       call gradout
!       call jggr
!       call clsgks
      
      x(1:ns) = MATMUL(sqrtPfU,z(1:ns)) + reshape(uback,(/ns/))
      x(ns+1:2*ns) = MATMUL(sqrtPfV,z(ns+1:2*ns)) + reshape(vback,(/ns/))
      x(2*ns+1:3*ns) = MATMUL(sqrtPfPhi,z(2*ns+1:3*ns)) + &
                       reshape(phiback,(/ns/))

      ! print *,x
      
      call stdir(x,blah)

      print *,'Final cost is: ',hjcou,', status:',status,', gnorm:',gnorm
      print *,'Final RMSE is: '

      do k=1,nt
          urmse = 0.
          vrmse = 0.
          phirmse = 0.

          do j=2,mm1
              do i=2,nm1
                  urmse = urmse + (u(i,j,k) - uexact(i,j,k))**2
                  vrmse = vrmse + (v(i,j,k) - vexact(i,j,k))**2
                  phirmse = phirmse + (phi(i,j,k) - phiexact(i,j,k))**2
              enddo
          enddo
          print *,'RMSE(',k,'):', sqrt(urmse)/rmseroot, &
                                  sqrt(vrmse)/rmseroot, &
                                  sqrt(phirmse)/rmseroot
      enddo

      open(unit=9,file='phi.dat',status='unknown', &
           form='formatted')
      write(9,*) ((zu(i,j,1),i=1,n),j=1,m),((zv(i,j,1),i=1,n),j=1,m), &
                 ((zphi(i,j,1),i=1,n),j=1,m)
      close(9)
      open(unit=10,file='hist.dat',status='unknown', &
           form='formatted')
      write(10,*) (((u(i,j,k),i=1,n),j=1,m),k=1,nt), &
                  (((v(i,j,k),i=1,n),j=1,m),k=1,nt), &
                  (((phi(i,j,k),i=1,n),j=1,m),k=1,nt)
      close(10)
      close(11)
!      open(unit=9,file='lbg.dat',status='unknown',
!     %     form='formatted')
!      write(9,*) ((u(i,j,1),i=1,n),j=1,m),((v(i,j,1),i=1,n),j=1,m),
!     1           ((phi(i,j,1),i=1,n),j=1,m)
!      close(9)
9999  stop
      end
!     ********************************************************

!    ****************************************************************


!     ********************************************************
      subroutine stdir(x,cost)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension x(nctr)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
      common /xlz/cost0

      call trsfd(x)
      do 2181 i=2,mm1
!     left boundary (l) i=1  beginning
      u(1,i,1)= u(nm1,i,1)
      v(1,i,1)= v(nm1,i,1)
      phi(1,i,1)= phi(nm1,i,1)
2181  continue
!     left  boundary (l)  end.

!     top boundary (t)  beginning  :
!      do 2182 i=2,nm1
      do 2182 i=1,n
      u(i,m,1)=   u(i,mm1,1)
      v(i,m,1)=   0.
      phi(i,m,1)= phi(i,mm1,1)
2182  continue
!     top boundary (t) end.
!     right boundary (r) beginning:
      do 2183 i=2,mm1
      u(n,i,1)= u(2,i,1)
      v(n,i,1)= v(2,i,1)
      phi(n,i,1)= phi(2,i,1)
2183  continue
!     right boundary (r) end.
!     bottom boundary (d) beginning:
!      do 2184 i=2,nm1
      do 2184 i=1,n
      u(i,1,1)= u(i,2,1)
      v(i,1,1)= 0.
      phi(i,1,1)= phi(i,2,1)

2184  continue
!     bottom  boundary (b) end.

      u(n,m,1)=(u(nm1,m,1)+u(n,mm1,1)+u(nm1,mm1,1))/3.
      v(n,m,1)=(v(nm1,m,1)+v(n,mm1,1)+v(nm1,mm1,1))/3.
      phi(n,m,1)=(phi(nm1,m,1)+phi(n,mm1,1)+phi(nm1,mm1,1))/3.
      u(n,1,1)=(u(nm1,1,1)+u(n,2,1)+u(nm1,2,1))/3.
      v(n,1,1)=(v(nm1,1,1)+v(n,2,1)+v(nm1,2,1))/3.
      phi(n,1,1)=(phi(nm1,1,1)+phi(n,2,1)+phi(nm1,2,1))/3.
      u(1,1,1)=(u(1,2,1)+u(2,1,1)+u(2,2,1))/3.
      v(1,1,1)=(v(1,2,1)+v(2,1,1)+v(2,2,1))/3.
      phi(1,1,1)=(phi(1,2,1)+phi(2,1,1)+phi(2,2,1))/3.
      u(1,m,1)=(u(1,mm1,1)+u(2,m,1)+u(2,mm1,1))/3.
      v(1,m,1)=(v(1,mm1,1)+v(2,m,1)+v(2,mm1,1))/3.
      phi(1,m,1)=(phi(1,mm1,1)+phi(2,m,1)+phi(2,mm1,1))/3.
      do 12 j=1,m
      do 13 i=1,n
      zu(i,j,1)=  u(i,j,1)
      zv(i,j,1)=  v(i,j,1)
      zphi(i,j,1)=phi(i,j,1)
13    continue
12    continue

141   continue
!     initialization of the cost-function cost
      costu=0.
      costv=0.
      costp=0.
      costupen=0.
      costvpen=0.
      costppen=0.
      do 2410 j=2,m-1
      do 2411 i=2,n-1

      call obs_operator(u(i,j,1),v(i,j,1),phi(i,j,1),uval,vval,phival)

      costu = costu &
              +(uval-uobs(i,j,1)) &
              *(uval-uobs(i,j,1))*wu(i,j)

      costv = costv &
              +(vval-vobs(i,j,1)) &
              *(vval-vobs(i,j,1))*wv(i,j)

      costp = costp &
              +(phival-phobs(i,j,1)) &
              *(phival-phobs(i,j,1))*wphi(i,j)
2411   continue
2410   continue
       cost=costu +costv +costp
2413   continue
2412   continue
      do 300 k=1,nt-1
      kp1=k+1
      km1=k-1
      if(k.eq.1)then
      kpass=k
      dt=ztemp
      else
      kpass=k-1
      dt=htemp
      endif
      do 200 j=2,mm1
      do 210 i=2,nm1
      ip1=i+1
      im1=i-1
      jp1=j+1
      jm1=j-1
      aa= (u(ip1,j,k)+u(im1,j,k))*(u(ip1,j,k)-u(im1,j,k))*hx
      bb= (v(i,jp1,k)+v(i,jm1,k))*(u(i,jp1,k)-u(i,jm1,k))*hy
      cc= (u(ip1,j,k)+u(im1,j,k))*(v(ip1,j,k)-v(im1,j,k))*hx
      dd= (v(i,jp1,k)+v(i,jm1,k))*(v(i,jp1,k)-v(i,jm1,k))*hy
      phix= (phi(ip1,j,k)-phi(im1,j,k))*hhx
      phiy= (phi(i,jp1,k)-phi(i,jm1,k))*hhy
      fu= f(i,j)*u(i,j,k)
      fv= f(i,j)*v(i,j,k)
      phiu= (phi(ip1,j,k)*u(ip1,j,k)-phi(im1,j,k)*u(im1,j,k))*hhx
      phiv= (phi(i,jp1,k)*v(i,jp1,k)-phi(i,jm1,k)*v(i,jm1,k))*hhy
!    ***************************************************************
      wzu(i,j)=aa+bb+phix-fv
      wzv(i,j)=cc+dd+phiy+fu
      wzphi(i,j)=phiu+phiv
!     ****************************************************************
      costu = costu +epnu*wzu(i,j)*wzu(i,j)
      costv = costv +epnv*wzv(i,j)*wzv(i,j)
      costp = costp +epnphi*wzphi(i,j)*wzphi(i,j)
!     ****************************************************************
!     ****************************************************************
      costupen = costupen +epnu*wzu(i,j)*wzu(i,j)
      costvpen = costvpen +epnv*wzv(i,j)*wzv(i,j)
      costppen = costppen +epnphi*wzphi(i,j)*wzphi(i,j)
!     ****************************************************************
      costpen=costupen + costvpen +costppen
!     ****************************************************************

!     ****************************************************************
       u(i,j,kp1)=   u(i,j,kpass)   -dt*(aa+bb+phix-fv)
       v(i,j,kp1)=   v(i,j,kpass)   -dt*(cc+dd+phiy+fu)
       phi(i,j,kp1)= phi(i,j,kpass) -dt*(phiu+phiv)
!     ****************************************************************
!    ***************************************************************
210   continue
200   continue
!     left boundary (l) i=1  beginning
      do 2081 i=2,mm1
      u(1,i,kp1)= u(nm1,i,kp1)
      v(1,i,kp1)= v(nm1,i,kp1)
      phi(1,i,kp1)= phi(nm1,i,kp1)
2081  continue
!     left  boundary (l)  end.

!     top boundary (t)  beginning  :
!      do 2082 i=2,nm1
      do 2082 i=1,n
      u(i,m,kp1)= u(i,mm1,kp1)
      v(i,m,kp1)= 0.
      phi(i,m,kp1)= phi(i,mm1,kp1)
2082  continue
!     top boundary (t) end.
!     right boundary (r) beginning:
      do 2083 i=2,mm1
      u(n,i,kp1)= u(2,i,kp1)
      v(n,i,kp1)= v(2,i,kp1)
      phi(n,i,kp1)= phi(2,i,kp1)

2083  continue
!     right boundary (r) end.
!     bottom boundary (d) beginning:
!      do 2084 i=2,nm1
      do 2084 i=1,n
      u(i,1,kp1)= u(i,2,kp1)
      v(i,1,kp1)= 0.
      phi(i,1,kp1)= phi(i,2,kp1)
2084  continue
!     bottom  boundary (b) end.
      u(n,m,kp1)=(u(nm1,m,kp1)+u(n,mm1,kp1)+u(nm1,mm1,kp1))/3.
      v(n,m,kp1)=(v(nm1,m,kp1)+v(n,mm1,kp1)+v(nm1,mm1,kp1))/3.
      phi(n,m,kp1)=(phi(nm1,m,kp1)+phi(n,mm1,kp1)+phi(nm1,mm1,kp1))/3.
      u(n,1,kp1)=(u(nm1,1,kp1)+u(n,2,kp1)+u(nm1,2,kp1))/3.
      v(n,1,kp1)=(v(nm1,1,kp1)+v(n,2,kp1)+v(nm1,2,kp1))/3.
      phi(n,1,kp1)=(phi(nm1,1,kp1)+phi(n,2,kp1)+phi(nm1,2,kp1))/3.
      u(1,1,kp1)=(u(1,2,kp1)+u(2,1,kp1)+u(2,2,kp1))/3.
      v(1,1,kp1)=(v(1,2,kp1)+v(2,1,kp1)+v(2,2,kp1))/3.
      phi(1,1,kp1)=(phi(1,2,kp1)+phi(2,1,kp1)+phi(2,2,kp1))/3.
      u(1,m,kp1)=(u(1,mm1,kp1)+u(2,m,kp1)+u(2,mm1,kp1))/3.
      v(1,m,kp1)=(v(1,mm1,kp1)+v(2,m,kp1)+v(2,mm1,kp1))/3.
      phi(1,m,kp1)=(phi(1,mm1,kp1)+phi(2,m,kp1)+phi(2,mm1,kp1))/3.
      do 2100 j=2,m-1
      do 2110 i=2,n-1

      call obs_operator(u(i,j,kp1),v(i,j,kp1),phi(i,j,kp1), &
                        uval,vval,phival)

      costu =   costu &
              +(uval-uobs(i,j,kp1)) &
              *(uval-uobs(i,j,kp1))*wu(i,j)

      costv =   costv &
              +(vval-vobs(i,j,kp1)) &
              *(vval-vobs(i,j,kp1))*wv(i,j)

      costp =   costp &
              +(phival-phobs(i,j,kp1)) &
              *(phival-phobs(i,j,kp1))*wphi(i,j)
2110   continue
2100   continue
      cost = costu + costv + costp
      do 2101 j=1,m
      do 2111 i=1,n
      zu(i,j,kp1)=  u(i,j,kp1)
      zv(i,j,kp1)=  v(i,j,kp1)
      zphi(i,j,kp1)=phi(i,j,kp1)

!      print 3103 ,i,j,kp1,zu(i,j,kp1),i,j,kp1,uobs(i,j,kp1),
!     %i,j,kp1,u(i,j,kp1)
3103  format (1x, 'zu('  ,i1, ',' ,i1, ',' ,i2,')= ',f8.2, &
               1x,'uobs(',i1, ',' ,i1, ',' ,i2,')= ',f8.2, &
               1x,'u(' ,i1, ',' ,i1, ',' ,i2,')= ',f8.2)
2111   continue
2101   continue

2113   continue
300   continue
!      print *,'Cost components:',costu,costv,costp,cost
      cost=cost/cost0
!       print 298,costu,costv,costp,cost
      iovar=6
!       write(iovar,298) costu,costv,costp,cost
298   format(1x,'value of the cost function: '/, &
             4x,'costu',6x,'costv',6x,'costp',6x,'cost',4x,/, &
             4(2x,e14.7))

!       write(iovar,398) costupen,costvpen,costpen,costpen
398   format(1x,'value of the cost function: '/, &
             4x,'costupen',4x,'costvpen',4x,'costpen',4x,'costpen',4x,/, &
             4(2x,e14.7))
!      iscomp=iscomp+1
!      xsjst(iscomp)=cost
      return
      end
!  second part
      subroutine stadj(grad)
!***********************************************************
!  in this subroutine the adjoint variables have the same
!  name  as the direct variables in the direct system
!  the adjoint code is derived by writing backward
!  the linear tangent model
!  the adjoint of each instruction is written step by step
!********************************************************
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension grad(nctr),awu(n,m),awv(n,m),awphi(n,m)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp

      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
      common /xlz/cost0
!**************************************************************
!   initialization of variables of the adjoint system
!**********************************************************
!      print 493
493   format(1x,'we enter in stadj')
      do 2 k=0,nt
      do 23 j=1,m
      do 24 i=1,n
      u(i,j,k)=   0.
      v(i,j,k)=   0.
      phi(i,j,k)= 0.
24    continue
23    continue
2     continue
      dju=1.
      djv=1.
      djphi=1.
      ntm1=nt-1
      do 300 k=ntm1,1,-1
      kp1=k+1
      km1=k-1
!****************************************************
!     adjoint operations on the cost function
!****************************************************

!      print 1991,dj,dju,djv,djphi
1991  format(1x,'dj= ',e14.7,1x,'dju= ',e14.7,1x,'djv= ',e14.7,1x, &
      'djphi= ',e14.7)
      do 2100 j=2,m-1
      do 2110 i=2,n-1
!     ********************************************************
!     update of the 13 11 89 : adding the adjoint of penalty terms
      awu(i,j)  =0.
      awv(i,j)  =0.
      awphi(i,j)=0.
      wzu(i,j)  =0.
      wzv(i,j)  =0.
      wzphi(i,j)=0.
!     end of this update part
!     ***********************************************************
      call obs_operator(zu(i,j,kp1),zv(i,j,kp1),zphi(i,j,kp1),uval, &
                        vval,phival)

      call obs_operator_adj(zu(i,j,kp1),zv(i,j,kp1),zphi(i,j,kp1),duval, &
                        dvval,dphival)

          u(i,j,kp1) =   u(i,j,kp1) &
                  + dju*2.*(uval-uobs(i,j,kp1))*duval*wu(i,j)

          v(i,j,kp1) =   v(i,j,kp1) &
                  + djv*2.*(vval-vobs(i,j,kp1))*dvval*wv(i,j)

          phi(i,j,kp1) = phi(i,j,kp1) &
                  + djphi*2.*(phival-phobs(i,j,kp1))*dphival*wphi(i,j)

!      if(i.ne.1.or.i.ne.n)go to 2110
!      if(j.ne.1.or.j.ne.n)go to 2110
!      print 3103 ,i,j,kp1,zu(i,j,kp1),i,j,kp1,u(i,j,kp1),
!     %i,j,kp1,wu(i,j)
3103  format (1x, 'zu('  ,i1, ',' ,i1, ',' ,i2,')= ',f8.2, &
               1x,'u(',i1, ',' ,i1, ',' ,i2,')= ',f8.2, &
               1x,'wu(' ,i1, ',' ,i1, ',' ,i2,')= ',e15.7)

2110   continue
2100   continue
!*****************************************************************
!     adjoint operations on the boundary
!****************************************************************
!     bottom  boundary (b) i=1  beginning
!      do 2084 i=2,nm1
      do 2084 i=1,n
      phi(i,2,kp1)= phi(i,2,kp1) + phi(i,1,kp1)
      u(i,2,kp1)=   u(i,2,kp1)   + u(i,1,kp1)
2084  continue
!     bottom   boundary (b)  end.
!     right boundary (r)  beginning  :
      do 2083 i=2,mm1
      phi(2,i,kp1)= phi(2,i,kp1) + phi(n,i,kp1)
      v(2,i,kp1)=     v(2,i,kp1) + v(n,i,kp1)
      u(2,i,kp1)=     u(2,i,kp1) + u(n,i,kp1)
2083  continue
!     right boundary (r) end.

!     top boundary (t) beginning:
!      do 2082 i=2,nm1
      do 2082 i=1,n
      phi(i,mm1,kp1)= phi(i,mm1,kp1) + phi(i,m,kp1)
      u(i,mm1,kp1)=     u(i,mm1,kp1) + u(i,m,kp1)
2082  continue
!     top boundary (t) end.
!     left boundary (l) beginning:
      do 2081 i=2,mm1
      phi(nm1,i,kp1)= phi(nm1,i,kp1) + phi(1,i,kp1)
      v(nm1,i,kp1)=     v(nm1,i,kp1) + v(1,i,kp1)
      u(nm1,i,kp1)=     u(nm1,i,kp1) + u(1,i,kp1)
2081  continue
!     left  boundary (l) end.
!*************************************************************
!     adjoint operations on the componants of the
!     trajectory
!*************************************************************
!     *******************************************************
      if(k.eq.1)then
      kpass=k
      dt=ztemp
      else
      kpass=k-1
      dt=htemp
      endif
!     *******************************************************
       do 200 j=2,mm1
       do 210 i=2,nm1
       ip1=i+1
       im1=i-1
       jp1=j+1
       jm1=j-1
!      uptdate of the 13 11 89 : adding adjoint of penalty terms
!    on recalcule le second membre du systeme direct stdir au temps k
!    we calculate again the second member of direct system at time k

      zaa= (zu(ip1,j,k)+zu(im1,j,k))*(zu(ip1,j,k)-zu(im1,j,k))*hx
      zbb= (zv(i,jp1,k)+zv(i,jm1,k))*(zu(i,jp1,k)-zu(i,jm1,k))*hy
      zcc= (zu(ip1,j,k)+zu(im1,j,k))*(zv(ip1,j,k)-zv(im1,j,k))*hx
      zdd= (zv(i,jp1,k)+zv(i,jm1,k))*(zv(i,jp1,k)-zv(i,jm1,k))*hy
      zphix= (zphi(ip1,j,k)-zphi(im1,j,k))*hhx
      zphiy= (zphi(i,jp1,k)-zphi(i,jm1,k))*hhy
      zfu= f(i,j)*zu(i,j,k)
      zfv= f(i,j)*zv(i,j,k)
      zphiu=(zphi(ip1,j,k)*zu(ip1,j,k)-zphi(im1,j,k)*zu(im1,j,k))*hhx
      zphiv=(zphi(i,jp1,k)*zv(i,jp1,k)-zphi(i,jm1,k)*zv(i,jm1,k))*hhy
!    ***************************************************************
      wzu(i,j)=zaa+zbb+zphix-zfv
      wzv(i,j)=zcc+zdd+zphiy+zfu
      wzphi(i,j)=zphiu+zphiv
!     ****************************************************************
!                     end of this update part
!      ***************************************************************
       phiu1=0.
       phiu2=0.
       phiv1=0.
       phiv2=0.
       aa1=0.
       aa2=0.
       bb1=0.
       bb2=0.
       phix=0.
       cc1=0.
       cc2=0.
       dd1=0.
       dd2=0.
       phiy=0.
       fu=0.
       fv=0.
!     *************************************************************
! 1
        phi(i,j,kpass)=phi(i,j,kpass)+phi(i,j,kp1)
        phiu1=phiu1-dt*phi(i,j,kp1)
        phiu2=phiu2-dt*phi(i,j,kp1)
        phiv1=phiv1-dt*phi(i,j,kp1)
        phiv2=phiv2-dt*phi(i,j,kp1)
! 2
        v(i,j,kpass)=v(i,j,kpass)+v(i,j,kp1)
        cc1=cc1-dt*v(i,j,kp1)
        cc2=cc2-dt*v(i,j,kp1)
        dd1=dd1-dt*v(i,j,kp1)
        dd2=dd2-dt*v(i,j,kp1)
        phiy=phiy-dt*v(i,j,kp1)
        fu=fu-dt*v(i,j,kp1)
! 3
        u(i,j,kpass)=u(i,j,kpass)+u(i,j,kp1)
        aa1=aa1-dt*u(i,j,kp1)
        aa2=aa2-dt*u(i,j,kp1)
        bb1=bb1-dt*u(i,j,kp1)
        bb2=bb2-dt*u(i,j,kp1)
        phix=phix-dt*u(i,j,kp1)
        fv=fv+dt*u(i,j,kp1)

!      *******************************************************
!      update of the 13 11 89 : adding adjoint of penalty terms
       awu(i,j)  =awu(i,j)   +   2.*epnu*wzu(i,j)*dju
       awv(i,j)  =awv(i,j)   +   2.*epnv*wzv(i,j)*djv
       awphi(i,j)=awphi(i,j) +   2.*epnphi*wzphi(i,j)*djphi
       aa1  =aa1  + awu(i,j)
       aa2  =aa2  + awu(i,j)
       bb1  =bb1  + awu(i,j)
       bb2  =bb2  + awu(i,j)
       phix =phix + awu(i,j)
       fv   =fv   - awu(i,j)
       cc1  =cc1  + awv(i,j)
       cc2  =cc2  + awv(i,j)
       dd1  =dd1  + awv(i,j)
       dd2  =dd2  + awv(i,j)
       phiy =phiy + awv(i,j)
       fu   =fu   + awv(i,j)
       phiu1 = phiu1 + awphi(i,j)
       phiu2 = phiu2 + awphi(i,j)
       phiv1 = phiv1 + awphi(i,j)
       phiv2 = phiv2 + awphi(i,j)
!      end of this update part
!      **********************************************************
! 4
       phi(i,jp1,k)=phi(i,jp1,k)+phiv2*zv(i,jp1,k)*hhy
       phi(i,jm1,k)=phi(i,jm1,k)-phiv2*zv(i,jm1,k)*hhy
! 5 (update du 24 04 89)
       phi(ip1,j,k)=phi(ip1,j,k)+phiu2*zu(ip1,j,k)*hhx
       phi(im1,j,k)=phi(im1,j,k)-phiu2*zu(im1,j,k)*hhx
! 6
       v(i,jp1,k)=v(i,jp1,k)+phiv1*zphi(i,jp1,k)*hhy
       v(i,jm1,k)=v(i,jm1,k)-phiv1*zphi(i,jm1,k)*hhy
! 7
       u(ip1,j,k)=u(ip1,j,k)+phiu1*zphi(ip1,j,k)*hhx
       u(im1,j,k)=u(im1,j,k)-phiu1*zphi(im1,j,k)*hhx
! 8
       v(i,j,k)=v(i,j,k)+fv*f(i,j)
! 9
       u(i,j,k)=u(i,j,k)+fu*f(i,j)
! 10
       phi(i,jp1,k)=phi(i,jp1,k)+phiy*hhy
       phi(i,jm1,k)=phi(i,jm1,k)-phiy*hhy
! 11
       phi(ip1,j,k)=phi(ip1,j,k)+phix*hhx
       phi(im1,j,k)=phi(im1,j,k)-phix*hhx
! 12  (update le 26 01 90)
       v(i,jp1,k)=v(i,jp1,k)+dd2*(zv(i,jp1,k)-zv(i,jm1,k))*hy
       v(i,jm1,k)=v(i,jm1,k)+dd2*(zv(i,jp1,k)-zv(i,jm1,k))*hy
! 13  (update le 26 01 90)
       u(ip1,j,k)=u(ip1,j,k)+cc2*(zv(ip1,j,k)-zv(im1,j,k))*hx
       u(im1,j,k)=u(im1,j,k)+cc2*(zv(ip1,j,k)-zv(im1,j,k))*hx
! 14 (update le 26 01 90)
       v(i,jp1,k)=v(i,jp1,k)+bb2*(zu(i,jp1,k)-zu(i,jm1,k))*hy
       v(i,jm1,k)=v(i,jm1,k)+bb2*(zu(i,jp1,k)-zu(i,jm1,k))*hy
! 15 (update le 26 01 90)
       u(ip1,j,k)=u(ip1,j,k)+aa2*(zu(ip1,j,k)-zu(im1,j,k))*hx
       u(im1,j,k)=u(im1,j,k)+aa2*(zu(ip1,j,k)-zu(im1,j,k))*hx
! 16 (update le 26 01 90)
       v(i,jp1,k)=v(i,jp1,k)+dd1*(zv(i,jp1,k)+zv(i,jm1,k))*hy
       v(i,jm1,k)=v(i,jm1,k)-dd1*(zv(i,jp1,k)+zv(i,jm1,k))*hy
! 17  (update le 26 01 90)
       v(ip1,j,k)=v(ip1,j,k)+cc1*(zu(ip1,j,k)+zu(im1,j,k))*hx
       v(im1,j,k)=v(im1,j,k)-cc1*(zu(ip1,j,k)+zu(im1,j,k))*hx
! 18  (update le 26 01 90)
       u(i,jp1,k)=u(i,jp1,k)+bb1*(zv(i,jp1,k)+zv(i,jm1,k))*hy
       u(i,jm1,k)=u(i,jm1,k)-bb1*(zv(i,jp1,k)+zv(i,jm1,k))*hy
! 19  (update le 26 01 90)
       u(ip1,j,k)=u(ip1,j,k)+aa1*(zu(ip1,j,k)+zu(im1,j,k))*hx
       u(im1,j,k)=u(im1,j,k)-aa1*(zu(ip1,j,k)+zu(im1,j,k))*hx

 210   continue
 200   continue
 300   continue
!***********************************************************
!    adjoint operations on the cost function at time 1
!***********************************************************
!      dju=dju+dj
!      djv=djv+dj
!      djphi=djphi+dj
      do 2510 j=2,m-1
      do 2511 i=2,n-1


      call obs_operator(zu(i,j,1),zv(i,j,1),zphi(i,j,1),uval, &
                        vval,phival)

      call obs_operator_adj(zu(i,j,1),zv(i,j,1),zphi(i,j,1),zuval, &
                        zvval,zphival)

      u(i,j,1) = u(i,j,1) &
               + dju*2.*(uval-uobs(i,j,1))*zuval*wu(i,j)

      v(i,j,1) = v(i,j,1) &
               + djv*2.*(vval-vobs(i,j,1))*zvval*wv(i,j)

      phi(i,j,1) = phi(i,j,1) &
                 + djphi*2.*(phival-phobs(i,j,1))*zphival*wphi(i,j)
      kp1=1
!      print 3107 ,i,j,kp1,zu(i,j,kp1),i,j,kp1,u(i,j,kp1),
!     %i,j,kp1,wu(i,j)
3107  format (1x, 'zu('  ,i1, ',' ,i1, ',' ,i2,')= ',f8.2, &
               1x,'u(',i1, ',' ,i1, ',' ,i2,')= ',f8.2, &
               1x,'wu(' ,i1, ',' ,i1, ',' ,i2,')= ',e15.7)

2511  continue
2510  continue
!*****************************************************************
!     adjoint operations on the boundary at time 1
!****************************************************************
!     bottom  boundary (b) i=1  beginning
!      do 2184 i=2,nm1
      do 2184 i=1,n
      phi(i,2,1)= phi(i,2,1) + phi(i,1,1)
      u(i,2,1)=   u(i,2,1)   + u(i,1,1)
2184  continue
!     bottom   boundary (b)  end.
!     right boundary (r)  beginning  :
      do 2183 i=2,mm1
      phi(2,i,1)= phi(2,i,1) + phi(n,i,1)
      v(2,i,1)=     v(2,i,1) + v(n,i,1)
      u(2,i,1)=     u(2,i,1) + u(n,i,1)
2183  continue
!     right boundary (r) end.
!     top boundary (t) beginning:
!      do 2182 i=2,nm1
      do 2182 i=1,n
      phi(i,mm1,1)= phi(i,mm1,1) + phi(i,m,1)
      u(i,mm1,1)=     u(i,mm1,1) + u(i,m,1)
2182  continue
!     top boundary (t) end.
!     left boundary (l) beginning:
      do 2181 i=2,mm1
      phi(nm1,i,1)= phi(nm1,i,1) + phi(1,i,1)
      v(nm1,i,1)=     v(nm1,i,1) + v(1,i,1)
      u(nm1,i,1)=     u(nm1,i,1) + u(1,i,1)
2181  continue
!     left  boundary (l) end.
      do 1367 j=2,nm1
      do 1368 i=2,nm1
      ui(i,j)  = u(i,j,1)
      vi(i,j)  = v(i,j,1)
      phii(i,j)= phi(i,j,1)
1368   continue
1367   continue
!      print 494
494   format(1x,'on passe grad')
      open(unit=11,file='ghist.dat',status='unknown', &
           form='formatted')
!      write(11,*) icall,(((u(i,j,k),i=1,n),j=1,m),k=1,nt),
!     1                  (((v(i,j,k),i=1,n),j=1,m),k=1,nt),
!     1                  (((phi(i,j,k),i=1,n),j=1,m),k=1,nt)
      call trsfr(grad)
      do 9832 i=1,nctr
      grad(i)=grad(i)/cost0
9832  continue
!      print 7028
7028  format(1x,'valeurs de grad')
      gnorm =0.
      do 7029 i=1,nctr
      gnorm=gnorm + grad(i)*grad(i)
!      print 7030,i,grad(i)
7030  format(1x,'grad(',i3,')= ',e14.7)
7029  continue
      gnorm=sqrt(gnorm)
!      gsst(iscomp)=alog10(gnorm)
!      print 7034,gnorm
      iovar=7
!      write(iovar,7034)gnorm
7034  format(1x,'euclidian norm of the gradient: ',e14.7)

      return
      end
!    ********************************************************
      subroutine initana
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
      data al/6.d+6/,dd/4.4d+6/,h0/2.d+3/, &
      gp/10./,fch/1.d-4/,beta/1.5d-11/
      data h1/-220./,h2/133./
! ***************************************************
!  this subroutine gives the initial values for the fields
!  wind components are deduced from the geopotential
!  the formula can be found in grammelvelt
!*******************************************************

      pi=3.14159
!      print 131
131   format(1x,'we re-enter in initana')
      dx=(2.*al)/(n-1)
      dy=(2.*dd)/(m-1)
      hx=1./(4.*dx)
      hy=1./(4.*dy)
      hhx=2.*hx
      hhy=2.*hy
      i1=162
      i2=91
      do 10 i=1,n
      do 20 j=1,m
      f(i,j)=1.d-4
      xix=(i-1)*dx/2.
      yiy=(j-1)*dy/2.
      co=9.*((dd/2.)-yiy)/(2.*dd)
!      co2=9.*((dd/2.)-yiy)/(2.*dd)
      aa=tanh(co)
!      aa2=15*tanh(co2)
      ab1=cosh(co)
!      ab2=15*cosh(co2)
      bb =1./(ab1*ab1)
!      bb2 =1./(ab2*ab2)
      triarg=2.*pi*xix/al
!      triarg2=2.*pi*xix/al
      xtrig=sin(triarg)
!      xtrig2=sin(triarg2)
      dtrig=(2.*pi/al)*cos(triarg)
!      dtrig2=(2.*pi/al)*cos(triarg2)
      fr=fch+beta*(yiy-dd/2.)
      dco=-4.5/dd
      derth=bb*dco
!      derth2=bb2*dco
      dersec=-2.*aa*derth
!      dersec2=-2.*aa2*derth2
      tmp1 = gp*(h0 + h1*aa + h2*bb*xtrig)
!      tmp2 = gp*(h0 + h1*aa2 + h2*bb2*xtrig2)

      phii(i,j)=tmp1
      vi(i,j)=gp*(h2*bb*dtrig)/fr
      ui(i,j)= -gp*(h1*derth + h2*xtrig*dersec )/fr
!      print 24,i,j,ui(i,j),i,j,vi(i,j),i,j,phii(i,j)
24    format (1x,'ui(',i2,',',i2,')=',e14.7, &
              2x,'vi(',i2,',',i2,')=',e14.7, &
              2x,'phii(',i2,',',i2,')=',e14.7)
20    continue
10    continue
      if(9.eq.9) return
! scale the variable
      end
      subroutine myvalue(hjcou,z,ndim)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension z(ndim),tabgr(ndim)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /xlz/cost0
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
      common /background/ uback(nm2,mm2),vback(nm2,mm2),phiback(nm2,mm2),& 
                          sqrtPfU(nm2*mm2,nm2*mm2),&
                          sqrtPfV(nm2*mm2,nm2*mm2),&
                          sqrtPfPhi(nm2*mm2,nm2*mm2)
      common /exact/ uexact(n,m,nt), vexact(n,m,nt), phiexact(n,m,nt)
      double precision x(ndim)
      double precision urmse, vrmse, phirmse
!      ndim=nctr
      ifin=1
      ns = nm2*mm2
      rmseroot = sqrt(1.*nm2*mm2)

!      x = z

      x(1:ns) = MATMUL(sqrtPfU,z(1:ns)) + reshape(uback,(/ns/))
      x(ns+1:2*ns) = MATMUL(sqrtPfV,z(ns+1:2*ns)) + reshape(vback,(/ns/))
      x(2*ns+1:3*ns) = MATMUL(sqrtPfPhi,z(2*ns+1:3*ns)) + &
                       reshape(phiback,(/ns/))

      hjcou = 0.
      call stdir(x,hjcou)

      urmse = 0.
      vrmse = 0.
      phirmse = 0.

      zdotz = dot_product(z,z)
      hjcou = hjcou + cost0*zdotz

      call trsfd(x)
      return
      end
      subroutine mygrad(tabgr,z,ndim)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension z(ndim),tabgr(ndim)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /xlz/cost0
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
      common /background/ uback(nm2,mm2),vback(nm2,mm2),phiback(nm2,mm2),& 
                          sqrtPfU(nm2*mm2,nm2*mm2),&
                          sqrtPfV(nm2*mm2,nm2*mm2),&
                          sqrtPfPhi(nm2*mm2,nm2*mm2)
      common /exact/ uexact(n,m,nt), vexact(n,m,nt), phiexact(n,m,nt)
      double precision x(ndim)
      double precision urmse, vrmse, phirmse

      ns = nm2*mm2

      x(1:ns) = MATMUL(sqrtPfU,z(1:ns)) + reshape(uback,(/ns/))
      x(ns+1:2*ns) = MATMUL(sqrtPfV,z(ns+1:2*ns)) + reshape(vback,(/ns/))
      x(2*ns+1:3*ns) = MATMUL(sqrtPfPhi,z(2*ns+1:3*ns)) + &
                       reshape(phiback,(/ns/))

      hjcou = 0.
      call stdir(x,hjcou)

      zdotz = dot_product(z,z)
      hjcou = hjcou + cost0*zdotz

      call stadj(tabgr)

      tabgr(1:ns) = MATMUL(sqrtPfU,tabgr(1:ns))
      tabgr(ns+1:2*ns) = MATMUL(sqrtPfV,tabgr(ns+1:2*ns))
      tabgr(2*ns+1:3*ns) = MATMUL(sqrtPfPhi,tabgr(2*ns+1:3*ns))

      tabgr = tabgr + cost0*2*z

      call trsfd(x)
      return
      end
      subroutine stdira
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi

      do 10 i=2,nm1
      do 11 j=2,nm1
      u(i,j,1)=   ui(i,j)
      v(i,j,1)=   vi(i,j)
      phi(i,j,1)= phii(i,j)
!      zu(i,j,1)=  u(i,j,1)
!      zv(i,j,1)=  v(i,j,1)
!      zphi(i,j,1)=phi(i,j,1)
11    continue
10    continue
      do 2181 i=2,mm1
!     left boundary (l) i=1  beginning
      u(1,i,1)=   u(nm1,i,1)
      v(1,i,1)=   v(nm1,i,1)
      phi(1,i,1)= phi(nm1,i,1)

2181  continue
!     left  boundary (l)  end.

!     right boundary (r) beginning:
      do 2183 i=2,mm1
      u(n,i,1)=   u(2,i,1)
      v(n,i,1)=   v(2,i,1)
      phi(n,i,1)= phi(2,i,1)

2183  continue
!     right boundary (r) end.
!     top boundary (t)  beginning  :
      do 2182 i=1,n
      u(i,m,1)=   u(i,mm1,1)
      v(i,m,1)=   0.
      phi(i,m,1)= phi(i,mm1,1)
2182  continue
!     top boundary (t) end.
!     bottom boundary (d) beginning:
      do 2184 i=1,n
      u(i,1,1)=   u(i,2,1)
      v(i,1,1)=   0.
      phi(i,1,1)= phi(i,2,1)

2184  continue
!     bottom  boundary (b) end.

      u(n,m,1)=(u(nm1,m,1)+u(n,mm1,1)+u(nm1,mm1,1))/3.
      v(n,m,1)=(v(nm1,m,1)+v(n,mm1,1)+v(nm1,mm1,1))/3.
      phi(n,m,1)=(phi(nm1,m,1)+phi(n,mm1,1)+phi(nm1,mm1,1))/3.
      u(n,1,1)=(u(nm1,1,1)+u(n,2,1)+u(nm1,2,1))/3.
      v(n,1,1)=(v(nm1,1,1)+v(n,2,1)+v(nm1,2,1))/3.
      phi(n,1,1)=(phi(nm1,1,1)+phi(n,2,1)+phi(nm1,2,1))/3.
      u(1,1,1)=(u(1,2,1)+u(2,1,1)+u(2,2,1))/3.
      v(1,1,1)=(v(1,2,1)+v(2,1,1)+v(2,2,1))/3.
      phi(1,1,1)=(phi(1,2,1)+phi(2,1,1)+phi(2,2,1))/3.
      u(1,m,1)=(u(1,mm1,1)+u(2,m,1)+u(2,mm1,1))/3.
      v(1,m,1)=(v(1,mm1,1)+v(2,m,1)+v(2,mm1,1))/3.
      phi(1,m,1)=(phi(1,mm1,1)+phi(2,m,1)+phi(2,mm1,1))/3.
!     initialization of the cost-function cost
!      costu=0.
!      costv=0.
!      costp=0.
      do 300 k=1,nt-1
      kp1=k+1
      km1=k-1
      if(k.eq.1)then
      kpass=k
      dt=ztemp
      else
      kpass=k-1
      dt=htemp
      endif
      do 200 i=2,n-1
      do 210 j=2,m-1
      ip1=i+1
      im1=i-1
      jp1=j+1
      jm1=j-1
      aa= (u(ip1,j,k)+u(im1,j,k))*(u(ip1,j,k)-u(im1,j,k))*hx
      bb= (v(i,jp1,k)+v(i,jm1,k))*(u(i,jp1,k)-u(i,jm1,k))*hy
      cc= (u(ip1,j,k)+u(im1,j,k))*(v(ip1,j,k)-v(im1,j,k))*hx
      dd= (v(i,jp1,k)+v(i,jm1,k))*(v(i,jp1,k)-v(i,jm1,k))*hy
      phix= (phi(ip1,j,k)-phi(im1,j,k))*hhx
      phiy= (phi(i,jp1,k)-phi(i,jm1,k))*hhy
      fu= f(i,j)*u(i,j,k)
      fv= f(i,j)*v(i,j,k)
      phiu= (phi(ip1,j,k)*u(ip1,j,k)-phi(im1,j,k)*u(im1,j,k))*hhx
      phiv= (phi(i,jp1,k)*v(i,jp1,k)-phi(i,jm1,k)*v(i,jm1,k))*hhy
!     ****************************************************************
      u(i,j,kp1)=   u(i,j,kpass)   -dt*(aa+bb+phix-fv)
      v(i,j,kp1)=   v(i,j,kpass)   -dt*(cc+dd+phiy+fu)
      phi(i,j,kp1)= phi(i,j,kpass) -dt*(phiu+phiv)
!     ****************************************************************
!     *****************************************************************

210   continue
200   continue
      do 2081 i=2,mm1
!     left boundary (l) i=1  beginning
      u(1,i,kp1)=   u(nm1,i,kp1)
      v(1,i,kp1)=   v(nm1,i,kp1)
      phi(1,i,kp1)= phi(nm1,i,kp1)

2081  continue
!     left  boundary (l)  end.

!     top boundary (t)  beginning  :
      do 2082 i=1,n
      u(i,m,kp1)=   u(i,mm1,kp1)
      v(i,m,kp1)=   0.
      phi(i,m,kp1)= phi(i,mm1,kp1)
2082  continue
!     top boundary (t) end.
!     right boundary (r) beginning:
      do 2083 i=2,mm1
      u(n,i,kp1)=   u(2,i,kp1)
      v(n,i,kp1)=   v(2,i,kp1)
      phi(n,i,kp1)= phi(2,i,kp1)

2083  continue
!     right boundary (r) end.
!     bottom boundary (d) beginning:
      do 2084 i=1,n
      u(i,1,kp1)=   u(i,2,kp1)
      v(i,1,kp1)=   0.
      phi(i,1,kp1)= phi(i,2,kp1)

2084  continue
!     bottom  boundary (b) end.

      u(n,m,kp1)=(u(nm1,m,kp1)+u(n,mm1,kp1)+u(nm1,mm1,kp1))/3.
      v(n,m,kp1)=(v(nm1,m,kp1)+v(n,mm1,kp1)+v(nm1,mm1,kp1))/3.
      phi(n,m,kp1)=(phi(nm1,m,kp1)+phi(n,mm1,kp1)+phi(nm1,mm1,kp1))/3.
      u(n,1,kp1)=(u(nm1,1,kp1)+u(n,2,kp1)+u(nm1,2,kp1))/3.
      v(n,1,kp1)=(v(nm1,1,kp1)+v(n,2,kp1)+v(nm1,2,kp1))/3.
      phi(n,1,kp1)=(phi(nm1,1,kp1)+phi(n,2,kp1)+phi(nm1,2,kp1))/3.
      u(1,1,kp1)=(u(1,2,kp1)+u(2,1,kp1)+u(2,2,kp1))/3.
      v(1,1,kp1)=(v(1,2,kp1)+v(2,1,kp1)+v(2,2,kp1))/3.
      phi(1,1,kp1)=(phi(1,2,kp1)+phi(2,1,kp1)+phi(2,2,kp1))/3.
      u(1,m,kp1)=(u(1,mm1,kp1)+u(2,m,kp1)+u(2,mm1,kp1))/3.
      v(1,m,kp1)=(v(1,mm1,kp1)+v(2,m,kp1)+v(2,mm1,kp1))/3.
      phi(1,m,kp1)=(phi(1,mm1,kp1)+phi(2,m,kp1)+phi(2,mm1,kp1))/3.
300   continue
      do 3100 k=1,nt
      do 3101 j=1,m
      do 3102 i=1,n
      zu(i,j,k)=  u(i,j,k)
      zv(i,j,k)=  v(i,j,k)
      zphi(i,j,k)=phi(i,j,k)
      call obs_operator(u(i,j,k),v(i,j,k),phi(i,j,k),uval,vval,phival)

      uobs(i,j,k)=uval
      vobs(i,j,k)=vval
      phobs(i,j,k)=phival

3102  continue
3101  continue
3100  continue
      return
      end


!   integration of the model and calculus of the cost function
! third part
      subroutine dirprt(xd,aui,avi,aphii,aub,avb,aphib)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension xd(nctr)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi
! ***************************************************
!  this subroutine gives perturbations for
!  the initial and boundary values of the fields whose
!  wind components are deduced from the geopotential
!  the formula can be found in grammelvelt
!*******************************************************
      pi=3.14159
      i1=162
      i2=91
      do 10 i=1,n
      do 20 j=1,m
!      phii(i,j)=zphi(i,j,1)
!      vi(i,j)=zv(i,j,1)
!      ui(i,j)= zu(i,j,1)
        oxu=rand()
        oxv=rand()
        oxp=rand()
       xu=dble(oxu)
       xv=dble(oxv)
       xp=dble(oxp)
       au=xu-0.5
       av=xv-0.5
       ap=xp-0.5
       u(i,j,1)=   au*aui*zu(i,j,1)
       v(i,j,1)=   av*avi*zv(i,j,1)
       phi(i,j,1)= ap*aphii*zphi(i,j,1)
!      print 5512,i,j,u(i,j,1)
!5512  format(1x,'u(,'i2, ',' ,i2, ',1)=', e14.7)

20    continue
10    continue
!    defining some boundary conditions
      i1=123
      i2=321
      do 21 k=1,nt
      do 22 i=2,mm1

        oxu=rand()
        oxv=rand()
        oxp=rand()
       xu=dble(oxu)
       xv=dble(oxv)
       xp=dble(oxp)

       au=xu-0.5
       av=xv-0.5
       ap=xp-0.5
        oxu=rand()
        oxv=rand()
        oxp=rand()
       xu=dble(oxu)
       xv=dble(oxv)
       xp=dble(oxp)
       au=xu-0.5
       av=xv-0.5
       ap=xp-0.5
22    continue
      do 23 i=2,nm1
        oxu=rand()
        oxv=rand()
        oxp=rand()
       xu=dble(oxu)
       xv=dble(oxv)
       xp=dble(oxp)
       au=xu-0.5
       av=xv-0.5
       ap=xp-0.5
        oxu=rand()
        oxv=rand()
        oxp=rand()
       xu=dble(oxu)
       xv=dble(oxv)
       xp=dble(oxp)
       au=xu-0.5
       av=xv-0.5
       ap=xp-0.5
23    continue
21    continue
      call trsfr(xd)

!************************************************************
!     commands for verifications
!************************************************************
      print 28
28    format(1x,'we exit from dirprt')
      return
      end
      subroutine perturb(x,hdir,y,eps,nn)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension x(1),hdir(1),y(1)
      do 1 i=1,nn
      y(i)=x(i)+eps*hdir(i)
1     continue
      return
      end
      subroutine ngdvr(x,hdir,y,grad,nn)
       implicit double precision (a-h),double precision (o-z)

      parameter (npt=14)
      dimension x(nn),y(nn),hdir(nn),grad(nn),talph(npt) &
               ,oxdra(npt),ozdra(npt),otdra(npt),dummy(nn)

      x = 0.

      call myvalue(hjcou,x,nn)
      call mygrad(grad,x,nn)

      hdir = grad/sqrt(dot_product(grad,grad))

       do 5014 i=1,nn
!       hdir(i)=1.d0
!      hdir(i)=0.d0
5014  continue
      print 5028
5028  format(1x,'values of grad and hdir')
      do 5029 i=1,nn
!      print 5030,i,grad(i),i,hdir(i)
5030  format(1x,'grad(',i3,')= ',e14.7, &
             2x,'hdir(',i3,')= ',e14.7 )
5029  continue

!      hpas=1.40d0
      hpas=10.
      val=1.
      do 846 i=1,npt
      talph(i)=val
      val=val/hpas
846   continue
      call scal(grad,hdir,nn,prod)
      do 4732 icour=1,npt
      alpha=talph(icour)
      call perturb(x,hdir,y,alpha,nn)

      call myvalue(hlcou,y,nn)

      grcal=alpha*prod
      hjacr=hlcou-hjcou
      alim=hjacr/grcal
      ozdra(icour)=alim
      oxdra(icour)=log10(alpha)
!     print 4717,alpha,i0,j0,k0
4717  format(1x,'perturbation=',1x,e14.7,1x,'at point:', &
                               '(',i2,',',i2,',',i2,')')
!      print 4733,i0,j0,k0,icour,hjacr,grcal
4733  format('pt:','(',i2,',',i2,',',i1,')','i=',i2, &
              1x,'hjacr=',e14.7,1x,'grcal=',e14.7)

      print *,'i=',icour,'alpha = ',alpha,', alpha test:',alim,'O(alpha):',(1-alim)/alpha
4734  format(1x,'i=',i2,1x,'estimed/calculated',1x,e14.7)
      xhjacr=hjacr-grcal
      alpeq=xhjacr/hjacr
      grdem=alpha*alpha
      equi=xhjacr/grdem

!      print 6172,icour,equi
!6172  format(1x, 'i= ',i2,1x,'equi=',1x,e14.7)
      abseq=abs(alpeq)
      otdra(icour)=log10(abseq)
4732  continue
!      call ezxy(oxdra,ozdra,npt,'verification of gradient $')
!      call ezxy(oxdra,otdra,npt,'existence  of second derivative $')
      return
      end
       subroutine scal(a,b,nn,prod)
       implicit double precision (a-h),double precision (o-z)

       dimension a(1),b(1)
       prod=0.d0
       do 1 i=1,nn
       prod= prod + a(i)*b(i)
1      continue
       print 3028,prod
3028   format(1x,'scalar product of grad by dir:',1x,e14.7)
       return
       end

      subroutine trsfr(x)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension      x(nctr)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi

      do 1370 j=2,mm1
      do 1371 i=2,nm1
      jidc1= i-1+(j-2)*nm2
      jidc2= jidc1+ nm2*mm2
      jidc3= jidc2+ nm2*mm2
      x(jidc1)= u(i,j,1)
      x(jidc2)= v(i,j,1)
      x(jidc3)= phi(i,j,1)
1371   continue
1370   continue
      return
      end

      subroutine trsfd(x)
       implicit double precision (a-h),double precision (o-z)

      parameter (n =21,m =n,nt = 80, &
        nm1= n-1,mm1= m-1,nm2= nm1-1,mm2= mm1-1, &
        nctr= 3*nm2*mm2, ntrav=25*nctr)
      dimension      x(nctr)
      common/tabx/ &
       u(n,m,0:nt),v(n,m,0:nt),phi(n,m,0:nt), &
      zu(n,m,0:nt),zv(n,m,0:nt),zphi(n,m,0:nt), &
      tu(n,m,0:nt),tv(n,m,0:nt),tphi(n,m,0:nt), &
      uobs(n,m,0:nt),vobs(n,m,0:nt),phobs(n,m,0:nt), &
      wu(n,m),wv(n,m),wphi(n,m),wt(nt), &
      ui(n,m),vi(n,m),phii(n,n),wzu(n,m),wzv(n,m),wzphi(n,m), &
      f(n,m),ufil(n,m),vfil(n,m),phifil(n,m)
      common /grpho/xijst(200),gist(200),xsjst(300),gsst(300),prod, &
      itcomp,iscomp
      common /scalx/hx,hy,hhx,hhy,htemp,ztemp,asscof,epnu,epnv,epnphi

      do 1370 j=2,nm1
      do 1371 i=2,mm1
      jidc1= i-1+(j-2)*mm2
      jidc2= jidc1+ nm2*mm2
      jidc3= jidc2+ nm2*mm2
      u(i,j,1)=x(jidc1)
      v(i,j,1)=x(jidc2)
      phi(i,j,1)=x(jidc3)
!      print 6621,i,j,u(i,j,1),jidc1,x(jidc1)
!6621  format(1x,'u(',i2,',',i2,',1)= ',f8.2,1x,'x(',i4,')=',1x,f8.2)
1371   continue
1370   continue
      return
      end
! fourth part minimization routine:lbfgs(liu and nocedal 1989)

